@@ -8,8 +8,6 @@ $p = pdl(-1.2,1);
 $x = pdl(0,0);
 $t = pdl(0,0);
 
-
-
 $st = '
 
 function mros
@@ -27,12 +25,10 @@ end jacobian
 
 ';
 
-#$h = levmar($p,$x,$t, FUNC => $st, MAXITS => 10000, NOCLEAN => 1);
-$h = levmar($p,$x,$t, CSRC => 'ros.c', MAXITS => 10000);
-print $h->{REPORT};
+$h = levmar($p,$x,$t, FUNC => $st, MAXITS => 1000, NOCLEAN => 1);
+
+print levmar_report($h);
 
-#$h = levmar( GETOPTS => 1);
-#print %$h;
 
 
 
